/*
   debug.c  -- handle debug and log method. It's part of pmail
   project. The TABLE STOP is set to 4 characters in this file.
   
   Copyright (C) 1998, 1999  Xuming <xuming@bigfoot.com>
   
   This program is free software; you can redistribute it and/or 
   modify it under the terms of the GNU General Public License as 
   published by the Free Software Foundation; either version 2, or 
   (at your option) any later version.
	   
   This program is distributed in the hope that it will be useful, 
   but WITHOUT ANY WARRANTY; without even the implied warranty of 
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License, the file COPYING in this directory, for
   more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/*
   debug.c
   
   This module exports some procedures about log file and debug 
   information output. 
   
   dbgInit()	open debug information output device and log file
   dbgClose()	close them, and release any allocted resources.
   
   dbgLevelIs()			these procedures access appropriate variety
   dbgMsgSendInto()		which enclosed in this module according to
   logFileLimitedIn()	the resource file .pmailrc.
   logFileNameIs()
   
   logPrintf()		save log information into log file
   dbgPrintf()		save debug information into debug file
   dbgPrintf2()		ditto, except refer to debug level argument
   
   errReport()		produce error message
   
   dbgVarietyExposure()  if defined macro name DEBUG, this procedure
   						 will display data structure of debug.

   For somewhat reason there is no 'stdarg.h' in my linux 
   distribution. I have to handle parameter table directly.
   This may cause some problem.
*/

#include	<stdio.h>
#include	<stdlib.h>
#include	<unistd.h>
#include	<errno.h>
#include	<time.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	"defs.h"
#include	"pmalloc.h"
#include	"socket.h"


static	int 	dbgLevel = 0;            /* data from .pmailrc */
static	char	*dbgMsgInto = NULL;      /* data from .pmailrc */

static	int 	logFileLimited = 10240;  /* data from .pmailrc */
static	char	*logFileIs = NULL;       /* data from .pmailrc */

static	int 	_dbgMsg = -1;
static	int 	_logFile = -1;


static char *timeStamp();

void dbgInit()
{
	char	buf[SVRBUF];
	struct	stat	fs;
	
	/* initiate the debug file, or device */
	
	if ( !dbgMsgInto )  _dbgMsg = -1;
	else if (!strcmp(dbgMsgInto, "stdout"))  _dbgMsg = 1;
	else if (!strcmp(dbgMsgInto, "stderr"))  _dbgMsg = 2;
	else if ( dbgLevel > 0 )  {
		getAbsolutePath(dbgMsgInto, buf);
		_dbgMsg = open(buf, O_WRONLY | O_CREAT, S_IRUSR | S_IWUSR);
		ftruncate(_dbgMsg, 0);
	}
	
	/* initiate the log file */
	
	if ( logFileIs )  {
		getAbsolutePath(logFileIs, buf);
		_logFile = open(buf, O_RDWR | O_APPEND | O_CREAT, 
				S_IREAD | S_IWRITE);	
		fstat(_logFile, &fs);
		if (logFileLimited && (fs.st_size > logFileLimited))  {
			ftruncate(_logFile, 0);
		}
	}
}


void dbgClose()
{
	if (_dbgMsg > 2)  {
		close(_dbgMsg);
		_dbgMsg = -1;
	}
	
	if (_logFile != -1)  {
		close(_logFile);
		_logFile = -1;
	}
	
	pfree(dbgMsgInto);
	pfree(logFileIs);
}




/* write information into specified file. If no file is specified
   in the resource file (.pmailrc), the requisition will be discard. */
	
void logPrintf(char *fmt, ... )
{
	char	buf[SVRBUF];
	void	*valist;
	
	if (_logFile > 0)  {
		sprintf(buf, "%s ", timeStamp());
		write(_logFile, buf, strlen(buf));
	
		valist = &fmt + 1;
		vsnprintf(buf, SVRBUF, fmt, valist);
		write(_logFile, buf, strlen(buf));
	}
}


/* write debug information into specified file if dbgLevel was set.
   If no file was open for writting, it will be discard. */

void dbgPrintf(char *fmt, ... )
{
	char	buf[SVRBUF];
	void	*valist;
	
	if ( !dbgLevel )  return;
	if ( _dbgMsg > 0 )  {
		sprintf(buf, "%s ", timeStamp());
		write(_dbgMsg, buf, strlen(buf));
		
		valist = &fmt + 1;
		vsnprintf(buf, SVRBUF, fmt, valist);
		write(_dbgMsg, buf, strlen(buf));
	}
}


/* ditto, except that it specifies a debug class. If debug level
   which specified in the resource file is less than this class, 
   it will do nothing but return at once. */

void dbgPrintf2(int level, char *fmt, ... )
{
	char	buf[SVRBUF];
	void	*valist;
	
	if ( dbgLevel < level )  return;
	if ( _dbgMsg > 0 )  {
		valist = &fmt + 1;
		vsnprintf(buf, SVRBUF, fmt, valist);
		write(_dbgMsg, buf, strlen(buf));
	}
}
	

/* this function produce an error message according to the error code
   and the attached message. Then transfer it into log file as well 
   as debug information output device */

void errReport(int flag, char *s)
{
	char	*cp;
	
	switch (flag)  {
		case MISHOSTS:
			cp = "%s: cannot resolve the host's name.\n";
			break;
		
		case MISSOCKET:
			cp = "%s: cannot open the socket.\n";
			break;
		
		case MISCONNECT:
			cp = "%s: fail to connect the host.\n";
			break;
		
		case MISANSWER:
			cp = "%s: server reply an unmarch answer.\n";
			break;

		case NOSMTPHOST:
			cp = "%s: no SMTP server is specified.\n";
			break;

		case LOGIN_FAIL:
			cp = "%s: access denied.\n";
			break;
		
		case LOOKERRNO:
			switch (errno)  {
				case EINTR:
					cp = "%s: interrupted by a signal before any data was read.\n";
					break;
			
				case EINVAL:
					cp = "%s: unable reading from the net object.\n";
					break;
			
				case EFAULT:
					cp = "%s: buffer is outside the accessible address space.\n";
					break;
					
				case EBADF:
					cp = "%s: invalid file description or not open.\n";
					break;
					
				case EPIPE:
					cp = "%s: reading end of socket is closed.\n";
					break;
					
				case ENOSPC:
					cp = "%s: no room for data.\n";
					break;
					
				default:
					cp = "%s: connection is lost.\n";
					break;
			}
			break;
	
		default:
			cp = "%s: unknown error flag.\n";	
	}
	logPrintf(cp, s);
	conPrintf(cp, s);
}
		  
	
static char *timeStamp()
{
	static	char	buf[32];
	char	*month[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
					"Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
	struct	tm	*t;
	time_t	secElaps;

	time(&secElaps);
	t = localtime(&secElaps);
	sprintf(buf, "%s %2d %02d:%02d:%02d", month[t->tm_mon], 
			t->tm_mday, t->tm_hour, t->tm_min, t->tm_sec);
	return buf;
}

/***********************************************************/

void dbgLevelIs(char *p)
{
	dbgLevel = atoi(p);
}

void dbgMsgSendInto(char *s)
{
	pfree(dbgMsgInto);
	dbgMsgInto = pmalloc(strlen(s)+1, NULL);
	strcpy(dbgMsgInto, s);
}

void logFileLimitedIn(char *p)
{
	logFileLimited = atoi(p);
	p = GotoLineEnd(p);
	switch (*p)  {
		case 'k':
		case 'K':
			logFileLimited *= 1024;
			break;
		case 'm':
		case 'M':
			logFileLimited *= 1024 * 1024;
			break;
	}
}

void logFileNameIs(char *s)
{
	pfree(logFileIs);
	logFileIs = pmalloc(strlen(s)+1, NULL);
	strcpy(logFileIs, s);
}


#ifdef	DEBUG

void  dbgVarietyExposure()
{
	dbgPrintf("Debug level is set to: [%d]\n", dbgLevel);
	dbgPrintf("Debug message send to: [%s][%d]\n", 
			dbgMsgInto, _dbgMsg);
	dbgPrintf("Log file is limited in: [%d]\n", logFileLimited);
	dbgPrintf("Log file is: [%s][%d]\n", logFileIs, _logFile);
}

#endif 
	

