
/*
   folder.c -- open mailbox for SMTP 
   The TABLE STOP is set to 4 characters in this file.
	
   Copyright (C) 1998, 1999  Xuming <xuming@bigfoot.com>
   
   This program is free software; you can redistribute it and/or 
   modify it under the terms of the GNU General Public License as 
   published by the Free Software Foundation; either version 2, or 
   (at your option) any later version.
	   
   This program is distributed in the hope that it will be useful, 
   but WITHOUT ANY WARRANTY; without even the implied warranty of 
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License, the file COPYING in this directory, for
   more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/*
   This module handle the mail folder for sending mail. It defines
   two method for open mail folder:
   
     openFolder(mbox, num)  open the specified mail box for sending
     createFolderFromCon()  handle the mail from console or stream
	 
   and two functions for access specified letter:
   
	 setActiveMail(num)           specifies an active letter
	 getMail(buf, len)            read the text of the active letter
     setMailStatus(num, status)   set the status of specified letter
	 
   finally, do not forget to close the folder:
   
     closeFolder()
*/
	 
#include	<stdio.h>
#include	<string.h>
#include	<unistd.h>
#include	<sys/stat.h>
#include	"defs.h"
#include	"smtp.h"
#include	"pmalloc.h"


#define     XMAILER \
"X-Mailer: pmail %s (Linux). Report bugs to <xuming@bigfoot.com>"


static	FILE	*_tmpfold = NULL;
static	char	*_tmpfilename = NULL;

static	FILE	*_fpipe = NULL;
static	char	*_pipename = NULL;

static	SENDTO	*_mfs = NULL;

static	char	*_mbox = NULL;



static char *handleMailHead(FILE *fp, SENDTO *st, char *buf, int len);
static int  readSignature(FILE *fout, int status);
static char *readMailBody(FILE *fp, SENDTO *st, char *buf, int len);
static char *handleMailBody(FILE *fp, SENDTO *st, char *buf, int len);
static SENDTO *insertSendmailRc();
static void addToAddress(SENDTO *st, char *to);
static void removeSendmailRc();
static void removeCopytoRc(COPYTO *to);
static int isheader(char *p);
static int isHeadInsideList(char *headlist, char *hdline);
static void addHeaderIntoList(char *headlist, char *hdline);
static int ismailstart(char *p);
static int folderBackup();
static int folderCompress();



int openFolder(char *mbox, int *num)
{
	FILE	*fp;
	SENDTO	*sendto;
	char	buf[SVRBUF];
	int 	mails;
	
	if (_tmpfold)  {
		conPrintf("Temporary mail folder is in active.\n");
		return OPENFILERR;
	}
	
	if (mbox == NULL)  {
		conPrintf("No outgoing mailbox specified.\n");
		*num = 0;
		return OK;
	}
	
	if ((fp = fopen(mbox, "rt")) == NULL)  {
		conPrintf("Cannot open mail folder - %s\n", mbox);
		return OPENFILERR;
	}
	getAbsolutePath(TMPFILE, buf);
	_tmpfilename = pmalloc(strlen(buf)+1, NULL);
	strcpy(_tmpfilename, buf);
	mktemp(_tmpfilename);
	if ((_tmpfold = fopen(_tmpfilename, "w+t")) == NULL)  {
		pfclose(fp);
		conPrintf("Cannot create temporary mail folder - %s\n", 
				_tmpfilename);
		return OPENFILERR;
	}

	
	/* preparing the original mailbox  */
	
	_mbox = pmalloc(strlen(mbox)+1, NULL);
	strcpy(_mbox, mbox);
	
	
	*buf  = '\0';
	mails = 0;
	while (!feof(fp))  {

		while(!ismailstart(buf))  {
			if (fgets(buf, SVRBUF, fp) == NULL)  goto allright;
		}
		sendto = insertSendmailRc();
		sendto->id = ++mails;
		sendto->origBoxSeek = ftell(fp);
		sendto->mailSeek = ftell(_tmpfold);
		fputs(buf, _tmpfold);
		
		if (handleMailHead(fp, sendto, buf, SVRBUF) == NULL)  {
			goto allright;
		}
		
		if (!ismailstart(buf))  {
			if (handleMailBody(fp, sendto, buf, SVRBUF) == NULL)  {
				goto allright;
			}
		}
	}
		
allright:
	pfclose(fp);
	*num = mails;
	return OK;
}


void setMailStatus(int num, int status)
{
	SENDTO	 *p;
	
	for (p = _mfs; p; p = p->next)  {
		if (p->id == num)  p->status = status;
	}
}


/* Note that num must start with 1 */

SENDTO *setActiveMail(int num)
{
	SENDTO	*p;
	char	buf[SVRBUF];
	
	for (p = _mfs; p; p = p->next)  {
		if (p->id == num)  {
			fseek(_tmpfold, p->mailSeek, SEEK_SET);
			if (fgets(buf, SVRBUF, _tmpfold) == NULL)  return NULL;
			break;
		}
	}
	return p;
}

char *getMail(char *buf, int len)
{
	if (fgets(buf, len, _tmpfold) == NULL)  return NULL;
	if (ismailstart(buf))  return NULL;
	return buf;
}


int createFolderFromCon()
{
	SENDTO	*st;
	char	buf[SVRBUF], headlist[SVRBUF], **to, *p;
	int 	i, num;
	
	*headlist = '\0';
	if (_tmpfold)  {
		conPrintf("Temporary mail folder is in active.\n");
		return OPENFILERR;
	}
	getAbsolutePath(TMPFILE, buf);
	_tmpfilename = pmalloc(strlen(buf)+1, NULL);
	strcpy(_tmpfilename, buf);
	
	mktemp(_tmpfilename);
	if ((_tmpfold = fopen(_tmpfilename, "w+t")) == NULL)  {
		conPrintf("Cannot create temporary mail folder - %s\n",
				_tmpfilename);
		pfree(_tmpfilename);
		return OPENFILERR;
	}
	
	st = insertSendmailRc();
	st->id			= 1;
	st->origBoxSeek = -1;
	st->status		= SMTP_NORMAL;
	st->mailSeek	= 0;
	
	fprintf(_tmpfold, "From - %s\n", timenow());
	if (isatty(0))  {
		if (smtpGetSubject() == NULL)  {
			fprintf(stderr, "Subject: ");
			fgets(buf, SVRBUF, stdin);
			chop(buf);
			if (!isblankline(buf))  smtpSetSubject(buf);
		}
		fprintf(stderr, "Use CTRL-D to terminate letter\n");
	}
	
	if ((p = smtpGetSender()) != NULL)  {
		fprintf(_tmpfold, "From: %s\n", p);
		st->from = pmalloc(strlen(p)+1, NULL);
		strcpy(st->from, p);
		addHeaderIntoList(headlist, p);
	}
	
	if ((to = smtpGetToaddr(&num)) != NULL)  {
		fprintf(_tmpfold, "To: %s", to[0]);
		addToAddress(st, to[0]);
		for (i = 1; i < num; i++)  {
			addToAddress(st, to[i]);
			fprintf(_tmpfold, ", %s", to[i]);
		}
		fprintf(_tmpfold, "\n");
		addHeaderIntoList(headlist, "To:");
	}
	
	if (smtpGetCc())  {
		addToAddress(st, smtpGetCc());
		fprintf(_tmpfold, "Cc: %s\n", smtpGetCc());
		addHeaderIntoList(headlist, "Cc:");
	}
	
	if (smtpGetBcc())  addToAddress(st, smtpGetBcc());

	if ((p = smtpGetSubject()) != NULL)  {
		fprintf(_tmpfold, "Subject: %s\n", p);
		addHeaderIntoList(headlist, "Subject:");
	}
	
	resetAppendixHeader();
	while ((p = getAppendixHeader()) != NULL)  {
		if ( isHeadInsideList(headlist, p))  continue;
		addHeaderIntoList(headlist, p);
		fprintf(_tmpfold, "%s\n", p);
	}
	
	if (!isHeadInsideList(headlist, XMAILER))  {
		fprintf(_tmpfold, XMAILER, PMAILVER);
		fprintf(_tmpfold, "\n");
	}
	
	if ((p = smtpGetSender()) != NULL)  {
		if (!isHeadInsideList(headlist, "Sender:"))  {
			fprintf(_tmpfold, "Sender: %s\n", p);
			addHeaderIntoList(headlist, "Sender:");
		}
	}
	
	fprintf(_tmpfold, "\n");

	*buf = '\0';
	handleMailBody(stdin, st, buf, SVRBUF);
	
	st->mailLen = ftell(_tmpfold) - st->mailSeek;
	return OK;
}



void closeFolder()
{
	int 	d;
	
	d = smtpGetDowithSent();
	if ((d != SMTP_DELETE) && (d != SMTP_BACKUP))  {
		pfclose(_tmpfold);
		goto close_ok;
	}
	
	if (d == SMTP_BACKUP)  {
		if (folderBackup())  goto close_ok;
	} 
	
	if (_mfs && (_mfs->origBoxSeek < 0))  {
		goto close_ok;
	}
	
	if ((d == SMTP_DELETE) || (d == SMTP_BACKUP))  folderCompress();
	
close_ok:
	
	if (_tmpfilename)  {
		unlink(_tmpfilename);
		free(_tmpfilename);
		_tmpfilename = NULL;
	}
	if (_pipename)  {
		unlink(_pipename);
		free(_pipename);
		_pipename = NULL;
	}
	pfree(_mbox);
	removeSendmailRc();
}



#ifdef	DEBUG

void folderVarietyExposure()
{
	SENDTO	*p;
	COPYTO	*ct;
	
	dbgPrintf("Temporary file is [%s]\n", _tmpfilename);
	for (p = _mfs; p; p = p->next)  {
		dbgPrintf("mail %d#, from [%s]\n", p->id, p->from);
		dbgPrintf("    length is %d, seek pointer is %d, status %d\n", 
				p->mailLen, p->mailSeek, p->status);
		for (ct = p->to; ct; ct = ct->next)  {
			dbgPrintf("    deliver to [%s]\n", ct->host);
		}
	}
}

#endif


/*********************************************************/

static char *handleMailHead(FILE *fp, SENDTO *st, char *buf, int len)
{
	int 	hf = S_COMMON;
	char	headlist[SVRBUF], tmp[SVRBUF], *p;
	
	*headlist = '\0';
	while(1) {
		if ((fgets(buf, len, fp)) == NULL)  return NULL;
		
		if (!isheader(buf))  break;
		else if (ismailfrom(buf))   hf = S_FROM;
		else if (ismailto(buf))     hf = S_TO;
		else if (ismailcc(buf))     hf = S_TO;
		else if (ismailbcc(buf))    hf = S_BCC;
		else if (ismailstatus(buf)) hf = S_STATUS;
		else if (*buf > ' ')        hf = S_COMMON;
		
		switch (hf)  {
			case S_FROM:
				pfree(st->from);
				st->from = pmalloc(strlen(buf), NULL);
				strcpy(st->from, &buf[5]);
				chop(st->from);
				break;
				
			case S_TO:
			case S_BCC:
				addToAddress(st, buf);
				break;
				
			case S_STATUS:
				p = &buf[strlen("status:")];
				p = SkipLeadSps(p);
				if (!strnicmp(p, "nopipe", 6))  {
					st->status = SMTP_NOPIPE;
				} else if (!strnicmp(p, "pure", 4))  {
					st->status = SMTP_PURE;
				}
				break;
				
			case S_COMMON:
				if (*buf > ' ')  addHeaderIntoList(headlist, buf);
				break;
		}
		
		/* the bcc line and status line must be erased */
		
		if ((hf != S_BCC) && (hf != S_STATUS))  {
			st->mailLen += strlen(buf);
			fputs(buf, _tmpfold);
		}
	}
	
	
	/* appending the alternative Copy To header line, which specified
	   in the command line arguments */
	
	if ((p = smtpGetCc()) != NULL)  {
		addToAddress(st, p);
		st->mailLen += strlen(p) + strlen("Cc: ") + 1;
		fprintf(_tmpfold, "Cc: %s\n", p);
	}
	if ((p = smtpGetBcc()) != NULL)  addToAddress(st, p);
	
	if ((p = smtpGetSubject()) != NULL)  {
		if (!isHeadInsideList(headlist, "Subject: "))  {
			addHeaderIntoList(headlist, "Subject: ");
			st->mailLen += strlen(p) + strlen("Subject: ") + 1;
			fprintf(_tmpfold, "Subject: %s\n", p);
		}
	}
			
	
	/* appending the appendix header which defined in the RC file,
	   while these appendix doesn't appear before */
	
	resetAppendixHeader();
	while ((p = getAppendixHeader()) != NULL)  {
		if (!isHeadInsideList(headlist, p))  {
			addHeaderIntoList(headlist, p);
			st->mailLen += strlen(p) + 1;
			fprintf(_tmpfold, "%s\n", p);
		}
	}
	
	sprintf(tmp, XMAILER, PMAILVER);
	if (!isHeadInsideList(headlist, tmp))  {
		addHeaderIntoList(headlist, tmp);
		st->mailLen += strlen(tmp) + 1;
		fprintf(_tmpfold, "%s\n", tmp);
	}
	
	if ((p = smtpGetSender()) != NULL)  {
		if (!isHeadInsideList(headlist, "Sender: "))  {
			addHeaderIntoList(headlist, "Sender:");
			st->mailLen += strlen(p) + strlen("Sender: ") + 1;
			fprintf(_tmpfold, "Sender: %s\n", p);
		}
	}
	
	fprintf(_tmpfold, "\n");
	st->mailLen++;
	
	return buf;
}


/* handle the mail body after handling the head. the key is 
   supporting the pipe */
	
static char *handleMailBody(FILE *fp, SENDTO *st, char *buf, int len)
{
	char	tmp[SVRBUF], *pt;
	int		cnt = 0;
	
	pt = smtpGetPipeto();
	if ( !pt || (st->status != SMTP_NORMAL))  {
		return readMailBody(fp, st, buf, len);
	}
	
	/* get temporary file name */
	
	getAbsolutePath(PIPETMP, tmp);
	_pipename = pmalloc(strlen(tmp)+1, NULL);
	strcpy(_pipename, tmp);
	if (!mktemp(_pipename))  {
		logPrintf("folder: create temporary file %s failed\n", tmp);
		pfree(_pipename);
		return readMailBody(fp, st, buf, len);
	}
	
	/* create the pipe */
		
	sprintf(tmp, "%s > %s", pt, _pipename);
	if ((_fpipe = popen(tmp, "w")) == NULL)  {
		logPrintf("folder: create pipe %s failed\n", tmp);
		pfree(_pipename);
		return readMailBody(fp, st, buf, len);
	}
		
	/* skip the leading blank line */
		
   	while (isblankline(buf))  {
		if (fgets(buf, len, fp) == NULL)  break;
	}
		
	/* now piping the mail body */
		
	do {
		/* if origBoxSeek is set to -1, the letter is read from 
		   console, so we must continuely read until EOF */
		
		if ((st->origBoxSeek != -1) && ismailstart(buf))  break;
		
		if (!strncmp(buf, "From ", 5))  fputc('>', _fpipe);
		fputs(buf, _fpipe);
		cnt += strlen(buf);
	} while (fgets(buf, len, fp) != NULL);
	cnt += readSignature(_fpipe, st->status);
	
	/* padding the pipe if there is the null mail body */
	
	if (cnt == 0)  fputs(" \n", _fpipe);
	
	/* copy the piped mail body back into the temporary mailbox */
	
	pclose(_fpipe);
	if ((_fpipe = fopen(_pipename, "rt")) != NULL)  {
		while(fgets(tmp, SVRBUF, _fpipe) != NULL)  {
			fputs(tmp, _tmpfold);
			st->mailLen += strlen(tmp);
		}
		st->mailLen ++;
		fprintf(_tmpfold, "\n");
		pfclose(_fpipe);
	}
	unlink(_pipename);
	pfree(_pipename);
	return buf;	/* buf, the last line which this function read */
				/* from outgoing mailbox */
}
	

/* read the body of a letter from 'fp', then store to temporary 
   mailbox, which opened as '_tmpfold'.*/
   
static char *readMailBody(FILE *fp, SENDTO *st, char *buf, int len)
{
	/* skip leading blank lines */

	while (isblankline(buf))  {
		if (fgets(buf, len, fp) == NULL)  {
			st->mailLen += readSignature(_tmpfold, st->status);
			return NULL;
		}
	}
	do {
		if ((st->origBoxSeek != -1) && ismailstart(buf))  break;
		if (!strncmp(buf, "From ", 5))  {
			fputc('>', _tmpfold);
			st->mailLen ++;
		}	
		fputs(buf, _tmpfold);
		st->mailLen += strlen(buf);
	} while (fgets(buf, len, fp) != NULL);
	
	len = readSignature(_tmpfold, st->status);
	st->mailLen += len;
	fprintf(_tmpfold, "\n");
	return buf;
}
		

/* append a signature file to 'fout'. The 'fout' may point to a 
   temporary mailbox or point to a pipe line, which depend on the
   caller.  Nevertheless if 'status' is set to SMTP_PURE, which 
   means do not piping this letter and do not appending anything, 
   the signature file will ignored. 
   
   function returns the signature's size
 */

static int  readSignature(FILE *fout, int status)
{
	FILE	*fsig;
	char	buf[SVRBUF], *sig;
	int		len = 0;
	
	sig = smtpGetSignature();
	if (sig && (status != SMTP_PURE))  {
		if ((fsig = fopen(sig, "rt")) == NULL)  {
			logPrintf("folder: cannot open signature file %s\n", sig);
			return 0;
		}
		*buf = '>', sig = buf + 1;
		while (fgets(sig, SVRBUF - 1, fsig))  {
			if (!strncmp(sig, "From ", 5))  {
				fputs(buf, fout);
				len += strlen(buf);
			} else {
				fputs(sig, fout);
				len += strlen(sig);
			}
		}
		pfclose(fsig);
	}
	return len;
}


/* Notes this procedure must be called by closeFolder() only !!
   copy backup-able letters into backup mailbox from temporary
   file. So these letters may be adjusted by pmail. 
   
   After backing up, the temporary file for sending mail has 
   finished its work. Though closes it. */

static int folderBackup()
{
	FILE	*bak;
	char	buf[SVRBUF], *bb;
	SENDTO	*st;
	int		rs = 0;
	
	/* check them for re-entering */
	
	if ( !_mfs || !_tmpfold )  return 0;
	
	bb = smtpGetBackupbox();
	if (bb == NULL)  return 1;
	if ((bak = fopen(bb, "at")) == NULL)  {
		logPrintf("folder: cannot open backup filebox\n");
		rs = 1;
	} else {
		for (st = _mfs; st; st = st->next)  {
			if (st->status != SMTP_BACKUP)  continue;
			fseek(_tmpfold, st->mailSeek, SEEK_SET);
			if (fgets(buf, SVRBUF, _tmpfold) == NULL)  break;
			fputs(buf, bak);
			while (getMail(buf, SVRBUF))  fputs(buf, bak);
		}
	}
	pfclose(bak);
	pfclose(_tmpfold);
	return rs;
}


/* Notes this procedure must be called by closeFolder() only !!
   some mails in the outgoing mailbox have sent, then deleted. Now 
   these removed letter ought to be cleaned, tnen leave untouched
   letter in the outgoing mailbox only.
   
   copy these untouched letter into a temporary file, then truncate
   outgoing mailbox to zero, and copy it back from temporary file.
   
   external varieties:
   char 	*_tmpfilename;	(constant)
   char		*_mbox; (constant)
   SENDTO	*_mfs;  (constant)
*/
   
static int folderCompress()
{
	FILE	*ftmp, *fbox;
	char	buf[SVRBUF];
	SENDTO	*st;
	int 	td;

	/* check the globel varieties for re-entering */
	
	if ( !_tmpfilename || !_mbox || !_mfs )  return 0;
	
	/* open temporary file for writting and outgoing mailbox for 
	   reading */
	
	ftmp = fopen(_tmpfilename, "w+t");
	if (ftmp == NULL)  return 1;
	
	fbox = fopen(_mbox, "r");
	if (fbox == NULL)  {
		fclose(ftmp);
		return 1;
	}
	
	
	/* copy untouched letter from outgoing mailbox to temporary file */
	
	for (st = _mfs; st; st = st->next)  {
		td = st->status;
		if ((td == SMTP_DELETE) || (td == SMTP_BACKUP))  continue;
		fseek(fbox, st->origBoxSeek, SEEK_SET);
		fprintf(ftmp, "From - %s\n", timenow());
		while(fgets(buf, SVRBUF, fbox))  {
			if (ismailstart(buf))  break;
			fputs(buf, ftmp);
		}
	}
	
	/* overthrow the outgoing mailbox by untouched letter */
	
	rewind(ftmp);
	fclose(fbox);
	if ((fbox = fopen(_mbox, "w")) != NULL)  {
		while(fgets(buf, SVRBUF, ftmp))  fputs(buf, fbox);
	}
	
	/* close the temporary file only. Thus the body of temporary file
	   must be cleaned by other procedure, such as the caller. */
	
	fclose(fbox);
	fclose(ftmp);
	return 0;
}


/**********************************************************/

static SENDTO *insertSendmailRc()
{
	SENDTO	*st;
	
	if (_mfs == NULL)  st = _mfs = pmalloc(sizeof(SENDTO), NULL);
	else  {
		st = _mfs;
		while (st->next)  st = st->next;
		st->next = pmalloc(sizeof(SENDTO), NULL);
		st = st->next;
	}
	st->from	= NULL;
	st->to		= NULL;
	st->next	= NULL;
	st->status	= SMTP_NORMAL;
	st->mailLen = 0;
	return st;
}

static void addToAddress(SENDTO *st, char *to)
{
	COPYTO	*new;
	char	buf[SVRBUF], *p;

	
	/* the parameter 'to' may start with 'Cc:' or 'Bcc:'.
	   So these marks must be skipped */
	
	if (ismailto(to) || ismailcc(to))  to += 3;
	else if (ismailbcc(to))  to += 4;
	strcpy(buf, to);
	
	if ((to = strtok(buf, ",")) == NULL)  return;
	do {
		if (isblankline(to))  continue;
		to = SkipLeadSps(to);
		
		new = pmalloc(sizeof(COPYTO), NULL);
		new->host = pmalloc(strlen(to)+1, NULL);
		strcpy(new->host, to);
		p = new->host + strlen(new->host) - 1;
		if (*p == '\n')  *p = '\0';
		
		new->next = st->to;
		st->to = new;
	} while ((to = strtok(NULL, ",")) != NULL);
}
		
	
static void removeSendmailRc()
{
	SENDTO	*st, *del;
	
	if (_mfs == NULL)  return;  		/* ready for re-enter */
	for (st = _mfs; st; )  {
		pfree(st->from);
		removeCopytoRc(st->to);
		del = st;
		st = st->next;
		pfree(del);
	}
	_mfs = NULL;
}

static void removeCopytoRc(COPYTO *to)
{
	COPYTO	*p, *del;
	
	if (to == NULL)  return;		/* for re-enter */
	for (p = to; p; )  {
		pfree(p->host);
		del = p;
		p = p->next;
		pfree(del);
	}
}

/*******************************************************/


/* each header line in a letter is looked as a line led of SPACE or 
   token before ':'. e.g.
   
   xxx: xxxxxxxxxxxxx                     - or -
     xxxxxxxxxxxx
	 
   xxx xxx: xxxxxxx   is not a header line.
*/

static int isheader(char *p)
{
	if (isblankline(p))  return NO;
	if (*p <= ' ')  return YES;
	for ( ; (*p > ' ') && (*p != ':'); p++);
	if (*p == ':')  return YES;
	return NO;
}



/* test if this head has introduced already, .e.g
	
	headlist = "From:To:Cc:X-mailer:Sender:"
	hdline = "X-mailer: pmail for linux"
	
   the answer is TRUE */

static int isHeadInsideList(char *headlist, char *hdline)
{
	char	buf[SVRBUF], *p;
	
	strcpy(buf, hdline);
	if ((p = strchr(buf, ':')) == NULL)  return NO;
	*(p+1) = '\0';
	
	p = headlist;
	while(1)  {
		if (!strncmp(p, buf, strlen(buf)))  return YES;
		if ((p = strchr(p, ':')) == NULL)  break;
		p++;
	}
	return NO;
}

static void addHeaderIntoList(char *headlist, char *hdline)
{
	char	buf[SVRBUF], *p;
	
	strcpy(buf, hdline);
	if ((p = strchr(buf, ':')) == NULL)  return;
	*(p+1) = '\0';
	strcat(headlist, buf);
}
	

/* the seperation mark of mailbox is so-called From-Space structure,
   which adopted by standard UNIX mail/mailx/elm. e.g:
   
   From xuming@bigfoot.com Tue Jan 26 02:35:07 1999       - or -
   From - Sat Feb 27 01:44:50 1999
*/
   
static int ismailstart(char *buf)
{
	char	*p;
	int		i;
	char	*week[] = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", 
						"Sat" };
	char	*mon[] = { "Jan", "Feb", "Mar",  "Apr", "May", "Jun", 
						"Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
	
	/* search "From " */
						
	p = buf;
	if (strncmp(p, "From ", 5))  return NO;
	
	/* search sender address "sender@domain" */
	
	if ((p = strchr(p, ' ')) == NULL)  return NO;
	p++;
	
	/* search week day */
	
	if ((p = strchr(p, ' ')) == NULL)  return NO;
	p++;
	for (i = 0; i < 7; i++)  if (!strncmp(p, week[i], 3))  break;
	if (i == 7)  return NO;
	
	/* search month */
	
	if ((p = strchr(p, ' ')) == NULL)  return NO;
	p++;
	for (i = 0; i < 12; i++)  if (!strncmp(p, mon[i], 3))  break;
	if (i == 12)  return NO;
	
	/* search date */
	
	if ((p = strchr(p, ' ')) == NULL)  return NO;
	p++;
	if (atoi(p) == 0)  return NO;
	
	/* search time */
	
	if ((p = strchr(p, ' ')) == NULL)  return NO;
	p++;
	if ((p = strchr(p, ':')) == NULL)  return NO;

	/* search year */

	if ((p = strchr(p, ' ')) == NULL)  return NO;
	p++;
	if (atoi(p) == 0)  return NO;
	
	return YES;
}
		


#ifdef		EXECUTIVE

/* gcc -DEXECUTIVE pmalloc.o misc.o debug.o -o folder folder.c */


char *smtpGetCc()  
{  
	static	char	*s = "abc, def";
	return s; 
}

char *smtpGetBcc()
{
	static	char	*s = "user1, user2";
	return s; 
}


static	int		idx = 0;

void resetAppendixHeader() 
{
	idx = 0;
}

char *getAppendixHeader()
{
	static	char	*s[] = {
		"UserHeader1: no content here!",
		"UserHeader2: no content here!",
		"UserHeader3: no content here!",
		"UserHeader4: no content here!",
		NULL
	};

	if (s[idx] == NULL)  return NULL;
	else  {
		idx ++;
		return s[idx-1];
	}
}

static	char	*subj = NULL;

void smtpSetSubject(char *s)
{
	subj = pmalloc(strlen(s)+1, NULL);
	strcpy(subj, s);
}

char *smtpGetSubject() 
{
	return subj; 
}

char **smtpGetToaddr(int *cnt)
{
	static	char *s[] = { "dest1", "dest2" };
	*cnt = 2;
	return s;
}		

char *smtpGetSender() 
{
	static char	*s = "no@sender";
	return s; 
}

char *smtpGetSignature() 
{
	static char	*s = "zzzzz";
	return s; 
}

char *smtpGetPipeto() 
{ 
	static char	*s = "mmencode -b";
	return s; 
}

char *smtpGetBackupbox()
{
	static char *s = "xbakup";
	return s;
}

int  smtpGetDowithSent()
{
	return SMTP_BACKUP;
}

int main(int argc, char **argv)
{
	int 	num = 1, i;
	SENDTO	*p;
	COPYTO	*ct;
	char	buf[SVRBUF];
	
	if (argc < 2)  {
		printf("Usage: folder mailbox\n");
		return 1;
	}
/*	openFolder(argv[1], &num);*/
	createFolderFromCon();
	
	printf("Temporary file is [%s]\n", _tmpfilename);
	for (p = _mfs; p; p = p->next)  {
		printf("mail %d#, from [%s]\n", p->id, p->from);
		printf("    length is %d, seek pointer is %d, status %d\n", 
				p->mailLen, p->mailSeek, p->status);
		for (ct = p->to; ct; ct = ct->next)  {
			printf("    deliver to [%s]\n", ct->host);
		}
	}
	fflush(stdout);
	
	while (1)  {
		printf("which one? (1-%d) ", num);
		fflush(stdout);
		gets(buf);
		
		i = atoi(buf);
		if (setActiveMail(i) == NULL)  break;
		while (getMail(buf, SVRBUF))  printf(buf);
		setMailStatus(i, SMTP_BACKUP);
	}

	closeFolder();
	return 0;
}

#endif


